
package test.net.sourceforge.pmd.symboltable;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import java.util.Iterator;
import java.util.List;
import java.util.Map;

import net.sourceforge.pmd.PMD;
import net.sourceforge.pmd.lang.ast.Node;
import net.sourceforge.pmd.lang.java.ast.ASTBlock;
import net.sourceforge.pmd.lang.java.ast.ASTCatchStatement;
import net.sourceforge.pmd.lang.java.ast.ASTEqualityExpression;
import net.sourceforge.pmd.lang.java.ast.ASTInitializer;
import net.sourceforge.pmd.lang.java.ast.ASTMethodDeclaration;
import net.sourceforge.pmd.lang.java.symboltable.Scope;
import net.sourceforge.pmd.lang.java.symboltable.VariableNameDeclaration;

import org.junit.Ignore;
import org.junit.Test;
public class AcceptanceTest extends STBBaseTst {

    @Ignore
    @Test
    public void testClashingSymbols() {
        parseCode(TEST1);
    }

    @Ignore
    @Test
    public void testInitializer() {
        parseCode(TEST_INITIALIZERS);
        ASTInitializer a = acu.findDescendantsOfType(ASTInitializer.class).get(0);
        assertFalse(a.isStatic());
        a = acu.findDescendantsOfType(ASTInitializer.class).get(1);
        assertTrue(a.isStatic());
    }

    @Ignore
    @Test
    public void testCatchBlocks() {
        parseCode(TEST_CATCH_BLOCKS);
        ASTCatchStatement c = acu.findDescendantsOfType(ASTCatchStatement.class).get(0);
        ASTBlock a = c.findDescendantsOfType(ASTBlock.class).get(0);
        Scope s = a.getScope();
        Map vars = s.getParent().getVariableDeclarations();
        assertEquals(1, vars.size());
        VariableNameDeclaration v = (VariableNameDeclaration)vars.keySet().iterator().next();
        assertEquals("e", v.getImage());
        assertEquals(1, ((List)vars.get(v)).size());
    }

    @Ignore
    @Test
    public void testEq() {
        parseCode(TEST_EQ);
        ASTEqualityExpression e = acu.findDescendantsOfType(ASTEqualityExpression.class).get(0);
        ASTMethodDeclaration method = e.getFirstParentOfType(ASTMethodDeclaration.class);
        Scope s = method.getScope();
        Map m = s.getVariableDeclarations();
        for (Iterator i = m.keySet().iterator(); i.hasNext();) {
            VariableNameDeclaration vnd = (VariableNameDeclaration)i.next();
            Node node = vnd.getNode();
            
        }
        
    }

    @Test
    public void testFieldFinder() {
        
        


    }

    @Ignore
    @Test
    public void testDemo() {
        parseCode(TEST_DEMO);
        System.out.println(TEST_DEMO);
        ASTMethodDeclaration node = acu.findDescendantsOfType(ASTMethodDeclaration.class).get(0);
        Scope s = node.getScope();
        Map m = s.getVariableDeclarations();
        for (Iterator i = m.keySet().iterator(); i.hasNext();) {
            VariableNameDeclaration d = (VariableNameDeclaration) i.next();
            System.out.println("Variable: " + d.getImage());
            System.out.println("Type: " + d.getTypeImage());
        }
    }


    private static final String TEST_DEMO =
            "public class Foo  {" + PMD.EOL +
            " void bar(ArrayList buz) { " + PMD.EOL +
            " } " + PMD.EOL +
            "}" + PMD.EOL;

    private static final String TEST_EQ =
            "public class Foo  {" + PMD.EOL +
            " boolean foo(String a, String b) { " + PMD.EOL +
            "  return a == b; " + PMD.EOL +
            " } " + PMD.EOL +
            "}" + PMD.EOL;

    private static final String TEST1 =
            "import java.io.*;" + PMD.EOL +
            "public class Foo  {" + PMD.EOL +
            " void buz( ) {" + PMD.EOL +
            "  Object o = new Serializable() { int x; };" + PMD.EOL +
            "  Object o1 = new Serializable() { int x; };" + PMD.EOL +
            " }" + PMD.EOL +
            "}" + PMD.EOL;

    private static final String TEST_INITIALIZERS =
            "public class Foo  {" + PMD.EOL +
            " {} " + PMD.EOL +
            " static {} " + PMD.EOL +
            "}" + PMD.EOL;

    private static final String TEST_CATCH_BLOCKS =
            "public class Foo  {" + PMD.EOL +
            " void foo() { " + PMD.EOL +
            "  try { " + PMD.EOL +
            "  } catch (Exception e) { " + PMD.EOL +
            "   e.printStackTrace(); " + PMD.EOL +
            "  } " + PMD.EOL +
            " } " + PMD.EOL +
            "}" + PMD.EOL;

    private static final String TEST_FIELD =
    "public class MyClass {" + PMD.EOL +
    " private int a; " + PMD.EOL +
    " boolean b = MyClass.ASCENDING; " + PMD.EOL +
    "}" + PMD.EOL;

    public static junit.framework.Test suite() {
        return new junit.framework.JUnit4TestAdapter(AcceptanceTest.class);
    }
}
